//==================================================================================
//
// OpenDCC - BiDiB for MobaLiSt
//
// Copyright (c) 2008 Wolfgang Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//----------------------------------------------------------------------------------
//
// file:      bidib_client_hardware.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// history:   2008-09-20 V0.01 kw  first version
//            2009-02-12 V0.02 kw  added definition for MFT
//
//----------------------------------------------------------------------------------
// If you want to use the xp_client:
//  --> please add your board definition below
//
//==================================================================================
// here we define the used hardware resources:
//
//   a) UART:   may be the only UART, UART0 or UART1, see below and header
//   b) PORT:   a port, which controls the direction setting.
//              This port must be initialized as output
//
//==================================================================================


#ifndef __BIDIB_CLIENT_HARDWARE_H__
#define __BIDIB_CLIENT_HARDWARE_H__

//==================================================================================
//
// Used Hardware
//----------------------------------------------------------------------------

#define UART   1
#define UART0  2                    // define some numbers (!=0)
#define UART1  3

#define TIMER0 10                   // define some numbers (!=0)
#define TIMER2 11 


#ifndef BOARD
 #warning SEVERE Warning - please define a BOARD
 #warning see bidib_client_if.c for explanations
 // use following statements in your config.h
 // #define BOARD_my_new_hardware 5
 // #define BOARD BOARD_my_new_hardware
#else

    // OpenDCC Command Station,
    // Port USART1
    #if (BOARD == BOARD_OPENDCC)
        #define BIDIB_UART             UART1                   // may be UART, UART0 or UART1
        #define HARDWARE_SET_BIDIB_RECEIVE    PORTB &= ~(1 << PB5)
        #define HARDWARE_SET_BIDIB_TRANSMIT   PORTB |= (1 << PB5)
    #endif

    // XP railcom bridge
    // USART0
    #if (BOARD == BOARD_RAILCOM_XP_BRIDGE)
        #define BIDIB_UART             UART0                   // may be UART, UART0 or UART1
        #define HARDWARE_SET_BIDIB_RECEIVE    PORTB &= ~(1 << PB7)
        #define HARDWARE_SET_BIDIB_TRANSMIT   PORTB |= (1 << PB7)
    #endif

    // XP melder
    // USART0
    #if (BOARD == BOARD_XP_MELDER)
        #define BIDIB_UART             UART0                   // may be UART, UART0 or UART1
        #define HARDWARE_SET_BIDIB_RECEIVE    PORTD &= ~(1 << PD2)
        #define HARDWARE_SET_BIDIB_TRANSMIT   PORTD |= (1 << PD2)
    #endif

    #if (BOARD == JEAN_MICHEL_FISCHER)
        #define BIDIB_UART             UART
        #define HARDWARE_SET_BIDIB_RECEIVE    PORTB &= ~(1 << PB2)
        #define HARDWARE_SET_BIDIB_TRANSMIT   PORTB |= (1 << PB2)
    #endif

    #if (BOARD == BOARD_MFT)               // Version 1.0 or 1.1 of MyFirstThrottle
        #define BIDIB_UART             UART0
        #define HARDWARE_SET_BIDIB_RECEIVE    { DISABLE_XP_TX; ENABLE_XP_RX; }
        #define HARDWARE_SET_BIDIB_TRANSMIT   { DISABLE_XP_RX; ENABLE_XP_TX; }
    #endif

    #if (BOARD == BOARD_GATEWAY0)          // Gateway, Xpressnet connected to UART0 (XP-USB)
        #define BIDIB_UART             UART0
        #define HARDWARE_SET_BIDIB_RECEIVE    { DISABLE_XP_TX; ENABLE_XP_RX; }
        #define HARDWARE_SET_BIDIB_TRANSMIT   { DISABLE_XP_RX; ENABLE_XP_TX; }
    #endif

    #if (BOARD == BOARD_GATEWAY1)          // Gateway, Xpressnet connected to UART1 (XP-XBEE Bridge)
        #define BIDIB_UART             UART1
        #define HARDWARE_SET_BIDIB_RECEIVE    { DISABLE_XP_TX; ENABLE_XP_RX; }
        #define HARDWARE_SET_BIDIB_TRANSMIT   { DISABLE_XP_RX; ENABLE_XP_TX; }
    #endif

    #if (BOARD == BOARD_BIDIB_S88)            // small S88 to BiDiB-Converter
        #define BIDIB_UART             UART0
        #define HARDWARE_SET_BIDIB_RECEIVE    { PORTD &= ~(1 << BIDIB_DIR); }
        #define HARDWARE_SET_BIDIB_TRANSMIT   { PORTD |= (1 << BIDIB_DIR); }
        #define HARDWARE_GET_RX               ( PIND & (1 << BIDIB_RX) )
    #endif

    #if (BOARD == BOARD_MOBALIST)            // small Controlboard for LEDs
        #if (__AVR_ATmega32__)
            #define BIDIB_UART             UART
        #elif (__AVR_ATmega644P__ || __AVR_ATmega644__)
            #define BIDIB_UART             UART0
        #else
            #warning: severe: no supported processor  
        #endif
        #define HARDWARE_SET_BIDIB_RECEIVE    { PORTD &= ~(1 << BIDIB_DIR); }
        #define HARDWARE_SET_BIDIB_TRANSMIT   { PORTD |= (1 << BIDIB_DIR); }
        #define HARDWARE_GET_RX               ( PIND & (1 << BIDIB_RX) )

    #endif



    #ifndef BIDIB_UART
        #warning SEVERE Warning - BOARD definition is void
    #endif

#endif  // ifndef BOARD

#endif  // __BIDIB_CLIENT_HARDWARE_H__
