//==================================================================================
//
// OpenDCC - BiDiB for MobaLiSt
//
//
// Copyright (c) 2012 Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//-----------------------------------------------------------------
//
// file:      bidib_client_if.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// history:   2008-07-11 V0.01 kw  started
//            2012-02-12 V0.02 kw  moved to bidib
//
//-----------------------------------------------------------------
//
// purpose:   BiDiB protocol for dcc
// content:   routines for BiDiBus-Client (RS485)
//
//-----------------------------------------------------------------


#ifndef __BIDIB_CLIENT_IF_H__
#define __BIDIB_CLIENT_IF_H__

#include <stdbool.h>


//-------------------------------------------------------------------------------
//
// useful typedefs for bidib messages (flat addressing)

typedef struct
  {
    unsigned char size;
    unsigned char node_addr;
    unsigned char index;
    unsigned char msg_type;
  } t_node_message_header;


typedef struct
  {
    t_node_message_header  header;
    unsigned char data;
  } t_node_message1;

typedef struct
  {
    t_node_message_header  header;
    unsigned char data[2];              // be sure, not to load more data here!
  } t_node_message2;

typedef struct
  {
    t_node_message_header  header;
    unsigned char data[10];
  } t_node_message10;

typedef struct
  {
    t_node_message_header  header;
    unsigned char data[18];
  } t_node_message18;

typedef struct
  {
    t_node_message_header  header;
    unsigned char data[28];
  } t_node_message28;


typedef enum
  {
    BIDIB_DISCONNECTED             = 0x00,  // nach Einschalten
    BIDIB_APPLIED                  = 0x01,  // Logon abgesendet, wir warten auf ACK
    BIDIB_CONNECTED                = 0x02,  // Verbunden
    BIDIB_REJECTED                 = 0x03,  // Abgewiesen
  } bidib_connect_t;

bidib_connect_t g_bidib_connect;

unsigned char g_bidib_backoff;                  // backoff during logon (loaded by disconnect)
unsigned char g_bidib_last_backoff;             // backoff in last run

unsigned char g_bidib_ca_time;                  // collision avoidance time during logon

#define BIDIB_SIZE_OF_LOGON_MSG  11             // Groesse der LOGON Message, diese besteht aus:
                                                // mlength, addr, mnum, type, unique_id(7)
                                                // Die ISR macht daraus:
                                                // plength, mlength, addr, mnum, type, unique_id(7) crc

unsigned char my_bidib_node_addr;           // our local address, where we are allowed to send

//=============================================================================
// Upstream Interface
//----------------------------------------------------------------------------
//
void init_bidib_client_if(void);                   // setup the interrupts and fifos

void stop_bidib_client_if(void);                   // disable further ints

void bidib_soft_stop (void);                       // sync. stop

void bidib_soft_start (void);                      // resume

void set_bidib_to_receive(void);                   // raw hw access - direction in

void set_bidib_to_transmit(void);                  // raw hw access - direction out

bool bidib_tx_fifo_put(unsigned char *new_message);
                                                    // message interface -here the main drops its
                                                    // message

void bidib_flush_tx(void);

void bidib_flush_rx(void);


// local

// bool bidib_send_byte (const unsigned char c);       // sends one byte with isr and fifo, bit 8 = 0

// bool bidib_is_all_sent(void);                       // true if fifo is empty and all data are sent


unsigned int bidib_rx_read (void);                  // reads one byte (including call bytes)
bool bidib_rx_ready (void);                         // true if one byte can be read

bool bidib_tx_fifo_healthy (void);                    // true if a complete message can bei wirtten




// debug only:

#define BIDIB_RX_BUF_SIZE  48              // mind. 16 / max 255
extern unsigned int bidib_rx_buf[BIDIB_RX_BUF_SIZE];


extern unsigned char bidib_rx_buf_read;        // point to next read
extern unsigned char bidib_rx_buf_write;       // point to next write
extern unsigned char bidib_rx_fill;

#if (DEBUG == 3)
  void X_push_to_rx(unsigned char d);
#endif



#endif  // __BIDIB_CLIENT_IF_H__

