//=================================================================================
//
// OpenDCC - BiDiB for MobaLiSt
//
// Copyright (c) 2010 Wolfgang Kufer
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//
//---------------------------------------------------------------------------------
//
// file:      bidib_client_parser.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// webpage:   http://www.opendcc.de
// history:   2012-04-12 V0.01 kw  start
//
//---------------------------------------------------------------------------------
//
// purpose:   BiDiB-Interface
//            
//            here: BiDiB client
//
//=================================================================================
// upstream interface (howto):
//
// to send something on bidib:
//   a) create message to be sent
//      (this must include size, addr, mnum, type and data
//      the crc checksum is calculated in the driver)
//   b) copy this message to bidib_tx_fifo (send_bidib_message)
//
// to receive a message:
//   a) all incoming messages are forwarded to Rx_Fifo
//   b) see bidib_parser for processing
//
// init:
//   a) be sure to set up timer correctly
//   b) define the usart and direction port
//   d) call init_bidib_client();
//
// run_time:
//   a) periodically call run_bidib_client(); (typically as task)
//
//=================================================================================

#include "bidib_messages.h"
#include "bidib_client_if.h"

//========================================================================

unsigned char g_bidib_identify;                 // identify mode

unsigned char g_bidib_spontan_enabled;          // the node is allowed to send occupancy / keystrokes


t_bidib_unique_id my_unique_id;



//========================================================================

void set_melderbits(unsigned char bits);   // FEATURE_BM_SIZE eintragen

//========================================================================



void init_bidib_client(void);
void restart_bidib_client(void);
void stop_bidib_client(void);

t_cr_task run_bidib_client(void);

void set_bidib_state(bidib_connect_t neu);      // Verbindungsstatus verwalten

void bidib_set_sys_identify(unsigned char state);   // Zustand darstellen
void bidib_send_sys_identify(unsigned char state);  // und absenden

bool bidib_send_onepara_msg(unsigned char msg_type, unsigned char dat);

unsigned char bidib_feature_key_send_isset(void);

unsigned char bidib_feature_macro_start_man_isset(void);

void bidib_send_keystroke(unsigned char key, unsigned char state);

//
// Simulation

void test_process_bidib_message(unsigned char *test);

//-- Accessory Support

void bidib_send_accessory_state(unsigned char acc, unsigned char fb_state, unsigned char wait);



