//===============================================================================
//
// OpenDCC - BiDiB for MobaLiSt
//           Simple Cooperative Task Scheduler
//
// Copyright (c) 2010 Wolfgang Kufer
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//
//-------------------------------------------------------------------------------
//
// file:      cortos_tasklist.c
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// webpage:   http://www.opendcc.de
// history:   2009-01-12 V0.01 kw  started
//
//-------------------------------------------------------------------------------
//
// purpose:   A small and tiny cooperative scheduler
//            This file contains the parts, which are individual for a project
//
//===============================================================================

#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>

#include "cortos.h"
#include "cortos_tasklist.h"
#include "timer.h"                  // access to systick

#if (DEBUG != 0)                    // generell debug allowed
    #define DEBUG_CORTOS   0        // 0: no debug messages
                                    // 1: LED turns on if not sleeping 
#endif


//------------------------------------------------------------------------------
//
// Task
//
//------------------------------------------------------------------------------
//
// Tasks (these are defined in other files, here defined again for the table)
//
// A task must return as soon as possible (cooperative multitasking), the return
// value determines the time gap until it is called again:
// return(-1) means: 'no longer ready'.
// return(5) means: 'ready again in 5 systick
//------------------------------------------------------------------------------
//
//                                      does:                   wakeup by:
t_cr_task task_idle (void);             // sleep                always
t_cr_task keyboard_task(void);          // check tracers        periodically
t_cr_task task_led_status(void);        // control leds         periodically
t_cr_task run_bidib_client(void);       // bidibus interf.      rx-interrupt

t_cr_task run_macro(void);              // macro interpreter    periodically
t_cr_task run_lights(void);             // dimming, blinking    periodically
t_cr_task event_handler(void);          // actions              key_ffio

// this is the list of all tasks
// first entry = lowest prio, last entry = highest prio

t_task tasklist[] =
   // id                     ready, wakeup, call
  {  [TASK_IDLE]          = {    1, 0,      task_idle},             // must be the first entry, lowest prio
     [TASK_EVENT_HANDLER] = {    0, 0,      event_handler},
     [TASK_KEYBOARD]      = {    1, 0,      keyboard_task},
     [TASK_LIGHTS]        = {    1, 0,      run_lights},
     [TASK_MACRO]         = {    1, 0,      run_macro},
     [TASK_LED]           = {    1, 0,      task_led_status},
     [TASK_BIDIB]         = {    0, 0,      run_bidib_client},      // highest prio
  };



// FIFO

// this is the MAIN FIFO!

cr_fifo_t key_fifo;
uint8_t key_fifo_buf[8];
