//------------------------------------------------------------------------
//
// OpenDCC 
//
// Copyright (c) 2009 Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
// 
//------------------------------------------------------------------------
//
// file:      datatypes.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// webpage:   http://www.opendcc.de
// history:   2009-09-14 V0.1 kw start
// 
//------------------------------------------------------------------------
//
// purpose:   useful types for AVR
//
//------------------------------------------------------------------------
//
//========================================================================
//
#ifndef _DATATYPES_H_
#define _DATATYPES_H_


typedef union
{
    uint16_t as_uint16;
    int16_t  as_int16;
    uint8_t  as_uint8[2];       // avr uses little endian - LSB is uint8[0]
    int8_t   as_int8[2];
    struct                      // if code runs on other endianess - change here
      {
        uint8_t  low8;
        uint8_t  high8;
      };
} t_data16;

typedef union
{
    uint32_t as_uint32;
    int32_t  as_int32;
    uint8_t  as_uint8[4];       // avr uses little endian - LSB is uint8[0]
    int8_t   as_int8[4];
    uint16_t  as_uint16[2];       // avr uses little endian - LSB is uint8[0]
    int16_t   as_int16[2];
    struct                      // if code runs on other endianess - change here
      {
        uint16_t  low16;
        uint16_t  high16;
      };
    struct                      // if code runs on other endianess - change here
      {
        uint8_t  low8;
        uint8_t  high8;
        uint8_t  upper8;
        uint8_t  highupper8;
      };
} t_data32;


//------------------------------------------------------------------------
//
// purpose:   useful types for model railway objects
//
//------------------------------------------------------------------------

typedef enum {DCC14 = 0, DCC27 = 1, DCC28 = 2, DCC128 = 3} t_format;

typedef struct
  {
    unsigned int addr;                  // address (either 7 or 14 bits)
    unsigned char speed;                // 0..28 or 0..126 (real speed)
    t_format format: 2;                 // 00 = 14, 01=27, 10=28, 11=128 speed steps.
                                        // DCC27 is not supported
    unsigned char dir: 1;               // 0 = forward, 1=revers
    unsigned char light: 1;             // function light
    unsigned char other: 1;             // 0: owned by me,   1: owned by other (upstream)
    unsigned char valid: 1;             // 0: not confirmed  1: record validated by CS, 
    unsigned char res: 2;               // reserved
    unsigned char slot;                 // owner (downstream)
    unsigned char f4_f1: 4;             // function 4 downto 1
    unsigned char f8_f5: 4;             // function 8 downto 5
    unsigned char f12_f9: 4;            // function 12 downto 9
    #if (DCC_F13_F28)
    unsigned char f16_f13: 4;           // function 16 downto 13
    unsigned char f20_f17: 4;           // function 20 downto 17
    unsigned char f24_f21: 4;           // function 24 downto 21
    unsigned char f28_f25: 4;           // function 28 downto 24
    #endif
  }  locomem_t;

typedef struct
  {
    unsigned int addr;                  // address (turnout addr)
    unsigned char aspect;               // 0..31 (multi aspect)  or 80..81 (dual aspect)
    unsigned char slot;                 // owner (downstream)
  }  trntmem_t;



typedef enum {STS_OFF           = 0,
              STS_ON            = 1,
              STS_STOP          = 2,
              STS_PROG          = 3,
              STS_PBUSY         = 4,
              STS_BUSY          = 5,
              STS_DISCONNECTED  = 6,
              STS_CMD_ERROR     = 7} t_status;


#define SIZE_LOCO_DATABASE    64         // no of locos in database (max. 250)


#endif   // _DATATYPES_H_
