//===============================================================================
//
// OpenDCC - BiDiB ---> MobaLiSt
//
// Copyright (c) 2012 Wolfgang Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//-------------------------------------------------------------------------------
//
// file:      event_handler.c
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// webpage:   http://www.opendcc.de
// history:   2012-04-14 V0.01 kw start
// 
//===============================================================================
//
// purpose:   BiDiB LightDecoder
//
//===============================================================================
//
//            here: central handling of events like keystrokes
//
//===============================================================================


#include <stdlib.h>
#include <stdbool.h>
#include <inttypes.h>
#include <avr/pgmspace.h>        // put var to program memory
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>

#include "config.h"
#include "hardware.h"
#include "cortos.h"
#include "timer.h"
#include "keyboard.h"               // event numbers
#include "led.h"

#include "bidib_client_parser.h"

#include "macro.h"
#include "event_handler.h"


void init_event_handler(void)
  {
    // already done in keyboard: cr_fifo_init (&key_fifo, key_fifo_buf, sizeof(key_fifo_buf));
    cr_fifo_set_comsumer (&key_fifo, TASK_EVENT_HANDLER);

    
  }


//------------------------------------------------------------------------------------------
// this task processes system event, these could be:
// external inputs
// keystrokes for identify
// special internal events (disconnect)

t_cr_task event_handler(void)
  {
    int temp;
    unsigned char event;
    
    temp = cr_fifo_get_nowait(&key_fifo);
    if (temp == -1) return(-1);                 // exit here, some body fooled us: 
                                                // we got woken, but there is no event in queue :-(
    event = (unsigned char) temp;
    
    switch(event)
      {
        case KEY_OPT1_PRESSED:
            if (bidib_feature_macro_start_man_isset())
              {
                macro_handle(0,BIDIB_MACRO_START);
              }

            if (bidib_feature_key_send_isset())
              {
                if (g_bidib_spontan_enabled) bidib_send_keystroke(0, 1);
              }
            break;
        case KEY_OPT2_PRESSED:
            if (bidib_feature_macro_start_man_isset())
              {
                macro_handle(1,BIDIB_MACRO_START);
              }

            if (bidib_feature_key_send_isset())
              {
                if (g_bidib_spontan_enabled) bidib_send_keystroke(1, 1);
              }
            break;
        case KEY_OPT1_RELEASED:

            if (bidib_feature_key_send_isset())
              {
                if (g_bidib_spontan_enabled) bidib_send_keystroke(0, 0);
              }
            break;
        case KEY_OPT2_RELEASED:
            if (bidib_feature_key_send_isset())
              {
                if (g_bidib_spontan_enabled) bidib_send_keystroke(1, 0); 
              }  
            break;
        case KEY_PROG_PRESSED:
            switch (g_bidib_identify)
              {
                case 1:
                    g_bidib_identify = 0;
                    bidib_set_sys_identify(0);          // Zustand darstellen
                    if (g_bidib_spontan_enabled && (g_bidib_connect == BIDIB_CONNECTED)) 
                      {
                        bidib_send_sys_identify(0);     // und absenden
                      }
                    break;
                default:
                case 0:             // enable / disable blink mode
                    if (g_bidib_connect == BIDIB_CONNECTED)
                      {
                        g_bidib_identify = 1;
                        bidib_set_sys_identify(1);          // Zustand darstellen
                        if (g_bidib_spontan_enabled) 
                          {
                            bidib_send_sys_identify(1);     // und absenden
                          }
                      }
                    else                            // Wenn gedrckt wird und wir sind disconnected
                      {                             // dann auch anzeigen.
                        // bidib_show_disconnected();
                        led_fast_blinking(LED_MSG);
                      }
                    break;
             }
            break;
        case KEY_CONNECT:
            if (g_bidib_connect == BIDIB_CONNECTED)
              {
                led_half(LED_PROG); 
              }
            else                            // Wenn gedrckt wird und wir sind disconnected
              {                             // dann auch anzeigen.
                // bidib_show_disconnected();
                led_fast_blinking(LED_MSG);
              }
            break;
        default:
            break;
      }
    return(1);  // ready again (in case there is already another event in the queue)
  }


