//===============================================================================
//
// OpenDCC - BiDiB for MobaLiSt
//
// Copyright (c) 2010 Wolfgang Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//-------------------------------------------------------------------------------
//
// file:      lights.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// webpage:   http://www.opendcc.de
// history:   2010-04-14 V0.01 kw start
//            2012-02-12 V0.02 kw ported to lightControl
//            2012-04-12 V0.03 kw ported to mobalist
// 
//===============================================================================
//
// purpose:   powerful light controller
//
//===============================================================================
//
// content:   routines for LED control (Outputport LED)
//            (we need CORTOS as scheduler)
//
//===============================================================================
//
#ifndef _LIGHTS_H_
#define _LIGHTS_H_

#include "bidib_messages.h"

#define NUM_OF_LIGHTS  24

#define FULL_BRIGHT 127


typedef struct
  {
    unsigned char state;        // Aktueller Status (0 = phase A, 1 = phase B)
    unsigned char rest;         // Zeit bis zum wechsel in Ticks 
    unsigned char volume;       // aktueller Istwert
    unsigned char target;       // aktueller Zielwert
    unsigned char delta;        // aktuelle Schrittweite

    unsigned char flashes;      // Anzahl Pulse (wie oft wird A angefahren)
    unsigned char act_flash;    // Counter dazu
    unsigned char pause;        // wenn != 0: pulse repetierend
    
    unsigned char time_A;       // Zeit in phase A (Einschaltzeit) 
    unsigned char light_A_val;  // aktueller Zielwert in Richtung A
    unsigned char delta_A;      // aktuelles Increment in Richtung A

    unsigned char time_B;       // Zeit in phase B (Ausschaltzeit)
    unsigned char light_B_val;  // aktueller Zielwert in Richtung B
    unsigned char delta_B;      // aktuelles Decrement in Richtung B
  } t_light;

extern t_light light[NUM_OF_LIGHTS];      // 24



#define LIGHT_TICK_PERIOD  10250L 
#if ((LIGHT_TICK_PERIOD / SYSTICK_PERIOD) == 0)
  #error: LIGHT_TICK_PERIOD too small!
#endif


extern void init_lights(void);                 // call at power up

t_cr_task run_lights(void);                  // call every light_TICK



// Light access / routines to manipulate the data structure for one light

void blink_light_fast(unsigned char light_enum);
void blink_light_middle(unsigned char light_enum);
void blink_light_slow(unsigned char light_enum);


void turn_light_off_fast(unsigned char light_enum);
void turn_light_off_soft(unsigned char light_enum);
void turn_light_on_soft(unsigned char light_enum);
void turn_light_on_fast(unsigned char light_enum);



typedef struct
  {
    unsigned char brightness_off;       // Helligkeit fr den Zustand AUS, Wertebereich 0..100
    unsigned char brightness_on;
    unsigned char dimm_off;             // Dimmzeit in Richtung AUS einstellen 0=schnell ... 255=langsam
    unsigned char dimm_on;              // Dimmzeit in Richtung EIN einstellen 0=schnell ... 255=langsam
  } t_light_cfg;


void set_light(unsigned char cmd, unsigned char port);
 
void config_light(t_bidib_lport_cfg* newcfg);

// return: 0: out of range
//         1: struct is filled
unsigned char config_light_query(t_bidib_lport_cfg* newcfg);

#endif // _LIGHTS_H_


