// OpenDCC - BiDiB for MobaLiSt

#ifndef _MACRO_H_
#define _MACRO_H_

#include "bidib_messages.h"     // get types

//-----------------------------------------------------------------------------


#define END_OF_MACRO    255     // das Macro wird beendet
#define START_MACRO     254
#define STOP_MACRO      253
#define BEGIN_CRITCAL   252     // das Marco darf nicht angehalten werden
#define END_CRITCAL     251     // das Macro darf angehalten werden (default)

#define FLAG_QUERY      250
#define FLAG_SET        249
#define FLAG_CLEAR      248

#define INPUT_QUERY1    247
#define INPUT_QUERY0    246
#define DELAY_RANDOM    245
#define DELAY_FIXED     244






typedef union
  {
    uint8_t byte;
    struct
      {
        uint8_t  cmd:4;
        uint8_t  type:4;
      };
  } t_lstate;


typedef struct                  // kurze Darstellung eines Schaltbefehls (im Macroform)
  {
    t_lstate lstate;
    uint8_t lvalue;
  }  t_macro_ctrl;




typedef struct
  {
    uint8_t delay;
    t_lstate lstate;
    uint8_t lvalue;
  }  t_macro_point;



typedef struct
  {
    uint8_t slow_down;              // this is the ratio between this cam and the cam tick.
    uint8_t repeat;                 // 0:forever, 1:once, 2...250 n times
    uint8_t tcode_min;              // Startpunkt 
    uint8_t tcode_hr;               // Startpunkt 
    uint8_t tcode_day;              // Startpunkt 
    t_macro_point item[NUM_OF_ITEMS_PER_MACRO];
  } t_macro;

t_macro macro[NUM_OF_MACROS];


// Umwandlungen von interner Darstellung auf Macrodarstellung und vv.

void convert_bidib2macro(t_bidib_port* bd_lc, t_macro_ctrl* m_lc);

void convert_macro2bidib( t_macro_ctrl* m_lc, t_bidib_port* bd_lc);

//-----------------------------------------------------------------------------
//  config interface

typedef struct
  {
    uint8_t  macro_i;
    uint8_t  index;
    uint8_t  delay;
    t_bidib_port port;
  }  t_macro_point_cfg;


typedef struct
  {
    uint8_t macro_i;
    uint8_t index;
    t_data32 value;
  }  t_macro_para;


void macro_set_point(t_macro_point_cfg* newpoint);

unsigned char macro_get_point(t_macro_point_cfg* newpoint);

unsigned char macro_handle(uint8_t macro_i, uint8_t opcode);

void macro_eval_timecode(uint8_t* tcode);

unsigned char macro_para_set(t_macro_para* newpara);

unsigned char macro_para_get(t_macro_para* newpara);

// einen Punkt ausfhren
void xperform(t_lstate lstate, uint8_t lvalue);


void init_macro(void);



// accessory mapper
void bidib_do_accessory(unsigned char acc_num, unsigned char aspect);

void init_accessory(void);


#endif  // _MACRO_H_
