//===============================================================================
//
// OpenDCC - BiDiB Support
//
// Copyright (c) 2010 Wolfgang Kufer
//
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//-------------------------------------------------------------------------------
//
// file:      prbs8.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// webpage:   http://www.opendcc.de
// history:   2011-12-17 V0.01 kw start
// 
//===============================================================================
//
// purpose:   backoff support for BiDiBus
//
//
//===============================================================================


extern unsigned char seed;             // var for prbs

unsigned char prbs8(void);



// linear feedback shift register (prbs)
//
//      |---|    |---|    |---|    |---|    |---|    |---|    |---|    |---|
//    ->| 0 |--->| 1 |-o->| 2 |-o->| 3 |-o->| 4 |--->| 5 |--->| 6 |--->| 7 |--o--->
//   |  |---|    |---| |  |---| |  |---| |  |---|    |---|    |---|    |---|  |
//   |                 |        |        |                                    |
//    <--------------- + <----- + <----- + <----------------------------------
//
static inline unsigned char static_prbs8(void)
       __attribute__((always_inline));

unsigned char static_prbs8(void)
  {
    unsigned char new_rnd;
    unsigned char temp;


    new_rnd = seed;                // copy bit 1
    new_rnd = new_rnd << 1;
    new_rnd = new_rnd ^ seed;      // xor bit 2
    new_rnd = new_rnd << 1;
    new_rnd = new_rnd ^ seed;      // xor bit 3
    new_rnd = new_rnd << 4;
    new_rnd = new_rnd ^ seed;      // xor bit 7

    // now put this bit to seed's lsb
    new_rnd = new_rnd >> 7;
    temp = seed << 1;
    seed = new_rnd + temp; 
    
    return(seed);
  }
